@@ -1,17 +1,16 @@
-# -*- coding: utf-8 -*-
 """ Handles reading and interpolation of atmopshere (grid) data """
 import itertools
 import logging
 
 import numpy as np
-from astropy import constants as const
 from scipy.interpolate import interp1d
 from scipy.optimize import curve_fit
 
-from ..large_file_storage import setup_atmo
-from .atmosphere import Atmosphere as Atmo
-from .atmosphere import AtmosphereError, AtmosphereGrid
+from astropy import constants as const
+
+from .atmosphere import Atmosphere as Atmo, AtmosphereError, AtmosphereGrid
 from .savfile import SavFile
+from ..large_file_storage import setup_atmo
 
 logger = logging.getLogger(__name__)
 
@@ -67,9 +66,7 @@ class AtmosphereInterpolator:
             if self.atmo_grid is None or self.source != atmo_grid:
                 atmo_file = self.lfs_atmo.get(atmo_grid)
                 self.source = atmo_grid
-                self.atmo_grid = SavFile(
-                    atmo_file, source=self.source, lfs=self.lfs_atmo
-                )
+                self.atmo_grid = SavFile(atmo_file)
         else:
             self.atmo_grid = atmo_grid
             self.source = atmo_grid.source
@@ -82,8 +79,6 @@ class AtmosphereInterpolator:
         else:
             atmo_grid = self.atmo_grid
 
-        assert len(atmo_grid) > 0, "No atmospheres for this geometry"
-
         # Get field names in ATMO and ATMO_GRID structures.
         depth = self.determine_depth_scale(self.depth, atmo_grid)
         interp = self.determine_interpolation_scale(self.interp, atmo_grid)
@@ -107,9 +102,7 @@ class AtmosphereInterpolator:
                 )
             else:
                 logger.info(
-                    "Input ATMO.GEOM='%s' overrides '%s' from grid.",
-                    self.geom,
-                    geom,
+                    "Input ATMO.GEOM='%s' overrides '%s' from grid.", self.geom, geom,
                 )
 
         # Add standard ATMO input fields, if they are missing from ATMO_IN.
@@ -129,8 +122,7 @@ class AtmosphereInterpolator:
         the mass column density or optical depth scale.
 
         How it works:
-
-        1. The second atmosphere is fitted onto the first, individually for
+        1) The second atmosphere is fitted onto the first, individually for
         each of the four atmospheric quantitites: T, xne, xna, rho.
         The fitting uses a linear shift in both the (log) depth parameter and
         in the (log) atmospheric quantity. For T, the midpoint of the two
@@ -138,9 +130,9 @@ class AtmosphereInterpolator:
         is used as initial guess for the other quantities. A penalty function
         is applied to each fit, to avoid excessively large shifts on the
         depth scale.
-        2. The mean of the horizontal shift in each parameter is used to
+        2) The mean of the horizontal shift in each parameter is used to
         construct the common output depth scale.
-        3. Each atmospheric quantity is interpolated after shifting the two
+        3) Each atmospheric quantity is interpolated after shifting the two
         corner models by the amount determined in step 1), rescaled by the
         interpolation fraction (frac).
 
@@ -180,51 +172,51 @@ class AtmosphereInterpolator:
             .xna  (vector[ndep]) atomic number density (1/cm^3)
             .rho  (vector[ndep]) mass density (g/cm^3)
         """
-        # """
-        # History
-        # -------
-        # 2004-Apr-15 Valenti
-        #     Initial coding.
-        # MB
-        #     interpolation on TAU scale
-        # 2012-Oct-30 TN
-        #     Rewritten to use either column mass (RHOX) or
-        #     reference optical depth (TAU) as vertical scale. Shift-interpolation
-        #     algorithms have been improved for stability in cool dwarfs (<=3500 K).
-        #     The reference optical depth scale is preferred in terms of interpolation
-        #     accuracy across most of parameter space, with significant improvement for
-        #     both cool models (where depth vs temperature is rather flat) and hot
-        #     models (where depth vs temperature exhibits steep transitions).
-        #     Column mass depth is used by default for backward compatibility.
-        # 2013-May-17 Valenti
-        #     Use frac to weight the two shifted depth scales,
-        #     rather than simply averaging them. This change fixes discontinuities
-        #     when crossing grid nodes.
-        # 2013-Sep-10 Valenti
-        #     Now returns an atmosphere structure instead of a
-        #     [5,NDEP] atmosphere array. This was necessary to support interpolation
-        #     using one variable (e.g., TAU) and radiative transfer using a different
-        #     variable (e.g. RHOX). The atmosphere array could only store one depth
-        #     variable, meaning the radiative transfer variable had to be the same
-        #     as the interpolation variable. Returns atmo.RHOX if available and also
-        #     atmo.TAU if available. Since both depth variables are returned, if
-        #     available, this routine no longer needs to know which depth variable
-        #     will be used for radiative transfer. Only the interpolation variable
-        #     is important. Thus, the interpvar= keyword argument replaces the
-        #     type= keyword argument. Very similar code blocks for each atmospheric
-        #     quantity have been unified into a single code block inside a loop over
-        #     atmospheric quantities.
-        # 2013-Sep-21 Valenti
-        #     Fixed an indexing bug that affected the output depth
-        #     scale but not other atmosphere vectors. Itop clipping was not being
-        #     applied to the depth scale ('RHOX' or 'TAU'). Bug fixed by adding
-        #     interpvar to vtags. Now atmospheres interpolated with interp_atmo_grid
-        #     match output from revision 398. Revisions back to 399 were development
-        #     only, so no users should be affected.
-        # 2014-Mar-05 Piskunov
-        #     Replicated the removal of the bad top layers in models
-        #     for interpvar eq 'TAU'
-        # """
+        """
+        History
+        -------
+        2004-Apr-15 Valenti
+            Initial coding.
+        MB
+            interpolation on TAU scale
+        2012-Oct-30 TN
+            Rewritten to use either column mass (RHOX) or
+            reference optical depth (TAU) as vertical scale. Shift-interpolation
+            algorithms have been improved for stability in cool dwarfs (<=3500 K).
+            The reference optical depth scale is preferred in terms of interpolation
+            accuracy across most of parameter space, with significant improvement for
+            both cool models (where depth vs temperature is rather flat) and hot
+            models (where depth vs temperature exhibits steep transitions).
+            Column mass depth is used by default for backward compatibility.
+        2013-May-17 Valenti
+            Use frac to weight the two shifted depth scales,
+            rather than simply averaging them. This change fixes discontinuities
+            when crossing grid nodes.
+        2013-Sep-10 Valenti
+            Now returns an atmosphere structure instead of a
+            [5,NDEP] atmosphere array. This was necessary to support interpolation
+            using one variable (e.g., TAU) and radiative transfer using a different
+            variable (e.g. RHOX). The atmosphere array could only store one depth
+            variable, meaning the radiative transfer variable had to be the same
+            as the interpolation variable. Returns atmo.RHOX if available and also
+            atmo.TAU if available. Since both depth variables are returned, if
+            available, this routine no longer needs to know which depth variable
+            will be used for radiative transfer. Only the interpolation variable
+            is important. Thus, the interpvar= keyword argument replaces the
+            type= keyword argument. Very similar code blocks for each atmospheric
+            quantity have been unified into a single code block inside a loop over
+            atmospheric quantities.
+        2013-Sep-21 Valenti
+            Fixed an indexing bug that affected the output depth
+            scale but not other atmosphere vectors. Itop clipping was not being
+            applied to the depth scale ('RHOX' or 'TAU'). Bug fixed by adding
+            interpvar to vtags. Now atmospheres interpolated with interp_atmo_grid
+            match output from revision 398. Revisions back to 399 were development
+            only, so no users should be affected.
+        2014-Mar-05 Piskunov
+            Replicated the removal of the bad top layers in models
+            for interpvar eq 'TAU'
+        """
 
         # Internal program parameters.
         min_drhox = min_dtau = 0.01  # minimum fractional step in RHOX
@@ -459,12 +451,10 @@ class AtmosphereInterpolator:
     def determine_depth_scale(self, depth, atmo_grid):
         """
         Determine ATMO.DEPTH radiative transfer depth variable. Order of precedence:
-
-        1. Value of ATMO_IN.DEPTH, if it exists and is set
-        2. Value of ATMO_GRID[0].DEPTH, if it exists and is set
-        3. 'RHOX', if ATMO_GRID.RHOX exists (preferred over 'TAU' for depth)
-        4. 'TAU', if ATMO_GRID.TAU exists
-
+            1. Value of ATMO_IN.DEPTH, if it exists and is set
+            2. Value of ATMO_GRID[0].DEPTH, if it exists and is set
+            3. 'RHOX', if ATMO_GRID.RHOX exists (preferred over 'TAU' for depth)
+            4. 'TAU', if ATMO_GRID.TAU exists
         Check that INTERP is valid and the corresponding field exists in ATMO.
 
         Parameters
@@ -503,19 +493,17 @@ class AtmosphereInterpolator:
             )
         if depth.lower() not in gtags:
             raise AtmosphereError(
-                "ATMO.DEPTH='{}', but ATMO. {} does not exist".format(depth, depth)
+                "ATMO.DEPTH='%s', but ATMO. %s does not exist" % (depth, depth)
             )
         return depth
 
     def determine_interpolation_scale(self, interp, atmo_grid):
         """
         Determine ATMO.INTERP interpolation variable. Order of precedence:
-
-        1. Value of ATMO_IN.INTERP, if it exists and is set
-        2. Value of ATMO_GRID[0].INTERP, if it exists and is set
-        3. 'TAU', if ATMO_GRID.TAU exists (preferred over 'RHOX' for interpolation)
-        4. 'RHOX', if ATMO_GRID.RHOX exists
-
+            1. Value of ATMO_IN.INTERP, if it exists and is set
+            2. Value of ATMO_GRID[0].INTERP, if it exists and is set
+            3. 'TAU', if ATMO_GRID.TAU exists (preferred over 'RHOX' for interpolation)
+            4. 'RHOX', if ATMO_GRID.RHOX exists
         Check that INTERP is valid and the corresponding field exists in ATMO.
 
         Parameters
@@ -554,7 +542,7 @@ class AtmosphereInterpolator:
             )
         if interp.lower() not in gtags:
             raise AtmosphereError(
-                "ATMO.INTERP='{}', but ATMO. {} does not exist".format(interp, interp)
+                "ATMO.INTERP='%s', but ATMO. %s does not exist" % (interp, interp)
             )
         return interp
 
@@ -655,11 +643,7 @@ class AtmosphereInterpolator:
             # Trace diagnostics.
             if self.verbose >= 5:
                 logger.info(
-                    "log(g) at [M/H]=%.3f: %.3f < %.3f < %.3f",
-                    mb[i],
-                    glo,
-                    logg,
-                    gup,
+                    "log(g) at [M/H]=%.3f: %.3f < %.3f < %.3f", mb[i], glo, logg, gup
                 )
 
         # End of loop through [M/H] bracket values.
@@ -794,29 +778,40 @@ class AtmosphereInterpolator:
             Interpolated atmosphere
         """
 
-        # We do this for every pair of atmosphere models
-        def interpolate(m0, m1, p, param, **kwargs):
+        # Interpolate 8 corner models to create 4 models at the desired [M/H].
+        atmo = np.empty((2, 2), dtype=object)
+        param = "monh"
+        p = monh
+
+        for (i, j) in np.ndindex(2, 2):
+            m0 = atmo_grid[icor[0, i, j]]
+            m1 = atmo_grid[icor[1, i, j]]
             p0 = getattr(m0, param)
             p1 = getattr(m1, param)
             pfrac = (p - p0) / (p1 - p0) if p0 != p1 else 0
-            return self.interp_atmo_pair(m0, m1, pfrac, interpvar=interp, **kwargs)
-
-        # Interpolate 8 corner models to create 4 models at the desired [M/H].
-        atmo = [[None, None], [None, None]]
-        for (i, j) in np.ndindex(2, 2):
-            m0 = atmo_grid[icor[0, j, i]]
-            m1 = atmo_grid[icor[1, j, i]]
-            atmo[i][j] = interpolate(m0, m1, monh, "monh", itop=itop)
+            atmo[i, j] = self.interp_atmo_pair(
+                m0, m1, pfrac, interpvar=interp, itop=itop
+            )
 
         # Interpolate 4 models at the desired [M/H] to create 2 models at desired
         # [M/H] and log(g).
         atmo2 = [None, None]
-        for k in range(2):
-            atmo2[k] = interpolate(atmo[k][0], atmo[k][1], logg, "logg")
+        param = "teff"
+        p = teff
+        for (k,) in np.ndindex(2):
+            m0 = atmo[0, k]
+            m1 = atmo[1, k]
+            p0 = getattr(m0, param)
+            p1 = getattr(m1, param)
+            pfrac = (p - p0) / (p1 - p0) if p0 != p1 else 0
+            atmo2[k] = self.interp_atmo_pair(m0, m1, pfrac, interpvar=interp)
 
         # Interpolate the 2 models at desired [M/H] and log(g) to create final
         # model at desired [M/H], log(g), and Teff
-        atmo3 = interpolate(atmo2[0], atmo2[1], teff, "teff")
+        t0 = atmo2[0].teff
+        t1 = atmo2[1].teff
+        tfrac = 0 if t0 == t1 else (teff - t0) / (t1 - t0)
+        atmo3 = self.interp_atmo_pair(atmo2[0], atmo2[1], tfrac, interpvar=interp)
 
         return atmo3
 
@@ -837,8 +832,7 @@ class AtmosphereInterpolator:
         atmo_grid : AtmosphereGrid
             complete atmosphere grid
         icor : array_like
-            indices for the corner models that were interpolated from the
-            atmosphere grid
+            indices for the corner models that were interpolated from the atmosphere grid
 
         Returns
         -------
